package oop6_dat6_u;

/**
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class Cylinder implements Comparable<IMyObject>, IMyObject
{

    private int number; //Nummer des Zylinders
    private int s1; //Durchmesser
    private int s2; //Höhe

    public Cylinder(int no, int a, int b)
    {
        this.number = no;
        this.s1 = a;
        this.s2 = b;
    }

    /**
     * Berechnet das Volumen
     * @return Volumen
     */
    public int getVolume()
    {
        return (s1 * s1 * (int) Math.PI * s2);
    }

    /**
     * Berechnet die Oberfläche des Kubus
     * @return Oberfläche
     */
    public int getSurface()
    {
        return (2 * (int) Math.PI * s1 * (s1 + s2));
    }

    /**
     * Liefert die grösste Seitenlänge als Ergebnis
     * @return grösse Seitenlänge
     */
    public int getMaxDimension()
    {
        if (s1 > s2) {
            return s1;
        } else {
            return s2;
        }
    }

    /**
     * String der Kugel
     * @return Beschreibung der Kugel
     */
    @Override
    public String toString()
    {
        /*
        return ("---------------------------------------\n"+
        "CYLINDER " + number + "\n" +
        "Radius: " + s1 + "\n" +
        "Höhe: " + s2 + "\n" +
        "Volume = " + this.getVolume() + "\n" +
        "Surface = " + this.getSurface()+ "\n");
         */
        return ("CYLINDER " + number + " | " +
            "Radius: " + s1 + " | " +
            "Höhe: " + s2 + " | " +
            "Volume = " + this.getVolume() + " | " +
            "Surface = " + this.getSurface() + "\n");
    }

    /**
     * vergleichen
     * @param other
     * @return Kubus gleich oder nicht
     */
    @Override
    public boolean equals(Object other)
    {
        if (this == other) // 1. Test auf Identität
        {
            return true;
        }
        if (other == null) // 2. Test auf null
        {
            return false;
        }
        if (other.getClass() != this.getClass())// 3. Test auf Vergleichbarkeit
        {
            return false;
        }
        if ( ! (this.getVolume() == ((Cylinder) other).getVolume()))// 4. Vergleich relev. Felder
        {
            return false;
        }
        return true;
    }

    /**
     * Hash-Code wird bassierend auf der (eindeutigen) Nummer definiert
     * @return Hash-Code
     */
    @Override
    public int hashCode()
    {
        return number;
    }

    /**
     * wird auf ihre Volumen verglichen
     * @param c zum Vergleichen
     * @return Differenz des Volumen
     */
    public int compareTo(IMyObject c)
    {
        return (this.getVolume() - c.getVolume());
    }

    /**
     * kleine "test" Methode
     * @param args
     */
    public static void main(String[] args)
    {
        Cylinder y1 = new Cylinder(1, 10, 10);
        Cylinder y2 = new Cylinder(2, 20, 10);
        Cylinder y3 = new Cylinder(3, 30, 5);

        System.out.println(y1.toString());
        System.out.println(y2.toString());
        System.out.println(y3.toString());

        System.out.println("y1 == y2 ? : " + y1.equals(y2));
        System.out.println("y2 == y3 ? : " + y2.equals(y3));
        System.out.println("y1 >  y2 ? : " + y1.compareTo(y2));
        System.out.println("y2 >  y3 ? : " + y2.compareTo(y3));
    }
}
